#![no_std]
#![feature(int_roundings)]
#![feature(lang_items)]
#![allow(internal_features)]

use drivers::vga::Vga13h;
use os::thunk::ThunkData;
use spin::Mutex;
use tetrisboard::TetrisBoard;

mod os;
mod tetrisboard;

#[macro_use]
mod drivers;

pub(crate) static VGA: Mutex<Vga13h> = Mutex::new(unsafe { Vga13h::new() });

#[no_mangle]
pub unsafe extern "C" fn start(
	_boot_disk: usize,
	thunk10: extern "C" fn(),
	_thunk13: extern "C" fn(),
	thunk15: extern "C" fn(),
	_thunk16: extern "C" fn(),
) -> ! {
	println!("Entered Rust, serial ready.");

	{
		// Set vga mode
		let mut data = ThunkData::new();
		data.eax = 0x13;
		data.with(thunk10);
	}

	{
		// Disable cursor
		let mut data = ThunkData::new();
		data.eax = 0x0100;
		data.ecx = 0x3F00;
		data.with(thunk10);
	}

	// Clear screen
	let mut v = VGA.lock();

	/*
	let fb = v.get_fb();
	let mut c: u8 = 0;
	for x in 0..128 {
		for y in 0..128 {
			let idx = Vga13h::pix_idx(x, y);
			fb[idx] = c;
			c = c.wrapping_add(1);
		}
	}
	*/

	let t = TetrisBoard::new();
	t.draw(&mut v);

	panic!("kernel");
}
